#!/usr/bin/expectk

# ferret.tcl -- the simple GUI to Ferret

# Copyright (C) 2000 by Moshe Cohen

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


# This tcl/tk script attempts to produce some sort of GUI
# to the Ferret with an ability for executing the basic commands 
# in graphical mode and all the other commands are still avaiable 
# from the command line. The target of this script was the work with
# oceanographic model, that is why the choise of features.

# Any suggestions/contributions would be recieved with gratitude, 
# though there is definitly no obligation to answer all the requests.

# This program was written using tcl/tk-8.0.5-30 and expect-5.28-30
# on Linux Red Hat 6.1 and tested on Ferret v. 5.01 for Linux. It 
# hasn't been tested on any other platform/versions, but there is 
# no reason for it not to work.


# You may contact the author via e-mail at a_mur@netvision.net.il or via
# Ferret Mailing list at Majordomo@ferret.wrc.noaa.gov.

proc Start {argc argv} {
    global spawn_id
    global tk_strictMotif
    set tk_strictMotif 0
    global sent_string
    
    global alreadyGotError
    set alreadyGotError 0

    wm title . "Ferret"
    #wm resizable . 0 1

    tk appname ferret

    frame .ftop -bd 5
    pack .ftop -side top -fill x  -expand false

    SetDatasetPane .ftop

    frame .fmain -bd 5 -relief sunken
    pack .fmain -side top -fill both -expand false
    SetMainPane .fmain

    frame .fbottom
    pack .fbottom -side bottom -fill x -expand false
    SetBottomPane .fbottom

    frame .fcommands
    pack .fcommands -side bottom -fill both -expand true
    SetCommandsPane .fcommands
    set t .fcommands.text



    log_user 0

# Ferret executable location, should be altered to ressemble your settings 

    spawn /usr/local/ferret/ferret

    set sent_string ""
    ExpectProgramOutput 0

}

proc SetDatasetPane {f} {
    label $f.l -text "Active Datasets" -relief ridge
    
    frame $f.listF

    set l $f.listF.list
    set sb $f.listF.sb
    listbox $l -height 5 -yscrollcommand "$sb set" -exportselection 0
    scrollbar $sb -command "$l yview"
    pack $sb -side right -fill y
    pack $l -expand no -fill x

    frame $f.buttons
    button $f.buttons.add -text "  Add..." -command "AddDatasetFile $l"
    button $f.buttons.remove -text " Remove " -command "DeleteDatasetFile $l" 
    button $f.buttons.activate -text Activate -command "ActivateDatasetFile $l" 
    bind $l <Double-Button-1> {ActivateDatasetFile %W}
    pack $f.buttons.add $f.buttons.remove $f.buttons.activate -side top -expand true -fill both

    pack $f.l  -side left -expand false  -anchor n -padx 5 -pady 5
    pack $f.listF -side left -expand true -fill both
    pack $f.buttons -side right -padx 5
}

proc SetBottomPane {f} {
    button $f.scriptB -text "Run Script..." -command RunScript
    button $f.quitB -text Quit -padx 5 -pady 5 -command executeCommands
    pack $f.scriptB -side left  -padx 5 -pady 5
    pack $f.quitB -side right
}

proc SetCommandsPane {f} {
    global sent_string
    global t
    
    set t $f.text
    set sb $f.sb
    text $t -relief sunken -bd 2 -height 10 -font 10x20 -bg black -fg white -insertbackground white -yscrollcommand "$sb set" 
    scrollbar $sb -command "$t yview"
    pack $sb -side right -fill y
    pack $t -expand yes -fill both
    
    bind $t <Return> {
	set sent_string [%W get endPrompt end]
	exp_send $sent_string
	ExpectProgramOutput 0
	break
    }
}
    

proc executeCommands {} {

    catch {eval exec rm -f [glob ferret.jnl.~*]}
    exit
    
}


proc ExpectProgramOutput {showUserInput} {
    global spawn_id
    global sent_string
    global t
    global ferret_output

    expect "yes? " {
	set s $expect_out(buffer)
	set ferret_output $s
	set l [string length $sent_string]

	if { $l > 0 && ! $showUserInput} {
	    # If not first prompt and echoing of user input was not requested,
	    # remove echoing of user's input
	    set s [string range $s [expr $l  + 1] end]
	    
	    # remove CRs
	    regsub -all "\r" $s "" s

	    $t insert end  "\n$s" 
	} else {
	    # remove CRs
	    regsub -all "\r" $s "" s
	    $t insert end $s
	}
	$t see end
	$t tag add readonly 1.0 {end -1 chars}
	$t tag conf readonly -foreground red
	$t mark set endPrompt {end -1 chars}
	$t mark gravity endPrompt left
	focus $t
    }
}

proc AddDatasetFile {listB} {
    set types {
	{"Data files"		{.cdf *.cdl}	}
	{"All files"		*}
    }
    set file [tk_getOpenFile -title "Choose Dataset" -filetypes $types -initialdir /POM/ferret]
    if {$file != ""} {
	$listB insert end $file
	$listB selection clear 0 end
	$listB selection set end
	$listB see end
    }
}
  
proc DeleteDatasetFile {listB} {
    set nn [$listB curselection]
    if {$nn != ""} {
	$listB delete $nn
    }
}

proc SetMainPane {f} {
    global typesList
    global title
    global prevFrameV
    global repeatIntervalV

    set title [label $f.title -text "<name>" -font 10x20]

    ###########
    set top [frame $f.top]

    label $top.a1 -text "Active Window"
    tk_optionMenu $top.a2 activeWindow 1 2 3 4

    checkbutton $top.o2 -text Overlay -variable overlayV

    label $top.f1 -text "Frame"
    tk_optionMenu $top.f2 frameV none upper lower left right
    set prevFrameV ""

    label $top.z1 -text "Zoom X"
    entry $top.z2 -width 4 -textvariable zoomV
    $top.z2 insert end 1

    pack $top.a1 $top.a2 -side left 
    pack $top.o2 -side left -padx 20
    pack $top.f1 $top.f2 -side left 
    pack $top.z2 $top.z1 -side right


##############
    
    set middle [frame $f.middle ]

    frame $middle.listF
    set l $middle.listF.list
    set typesList $l
    set sb $middle.listF.sb
    listbox $l -height 5 -yscrollcommand "$sb set" -exportselection 0
    scrollbar $sb -command "$l yview"
    pack $sb -side right -fill y
    pack $l -expand no -fill x

    bind $l <ButtonRelease-1> TypeChanged


    set activeVarsF [frame $middle.activeVars ]

    frame $activeVarsF.xF
    label $activeVarsF.xF.xL -text "X              " -textvariable Xlimits -font fixed
    entry $activeVarsF.xF.xFromE -textvariable fromX -width 15
    label $activeVarsF.xF.xTo -text "to"
    entry $activeVarsF.xF.xToE -textvariable toX -width 15
    pack $activeVarsF.xF.xL $activeVarsF.xF.xFromE $activeVarsF.xF.xTo $activeVarsF.xF.xToE -side left
    bind $activeVarsF.xF.xFromE <KeyRelease> {set toX $fromX}

    frame $activeVarsF.yF
    label $activeVarsF.yF.yL -text "Y              " -textvariable Ylimits -font fixed
    entry $activeVarsF.yF.yFromE -textvariable fromY -width 15
    label $activeVarsF.yF.yTo -text "to"
    entry $activeVarsF.yF.yToE -textvariable toY -width 15
    pack $activeVarsF.yF.yL $activeVarsF.yF.yFromE $activeVarsF.yF.yTo $activeVarsF.yF.yToE -side left
    bind $activeVarsF.yF.yFromE <KeyRelease> {set toY $fromY}

    frame $activeVarsF.zF
    label $activeVarsF.zF.zL -text "Z              " -textvariable Zlimits -font fixed
    entry $activeVarsF.zF.zFromE -textvariable fromZ -width 15
    label $activeVarsF.zF.zTo -text "to"
    entry $activeVarsF.zF.zToE -textvariable toZ -width 15
    pack $activeVarsF.zF.zL $activeVarsF.zF.zFromE $activeVarsF.zF.zTo $activeVarsF.zF.zToE -side left
    bind $activeVarsF.zF.zFromE <KeyRelease> {set toZ $fromZ}

    frame $activeVarsF.tF
    label $activeVarsF.tF.tL -text "T              " -textvariable Tlimits -font fixed
    entry $activeVarsF.tF.tFromE -textvariable fromT -width 15
    label $activeVarsF.tF.tTo -text "to"
    entry $activeVarsF.tF.tToE -textvariable toT -width 15
    pack $activeVarsF.tF.tL $activeVarsF.tF.tFromE $activeVarsF.tF.tTo $activeVarsF.tF.tToE -side left
    bind $activeVarsF.tF.tFromE <KeyRelease> {set toT $fromT}


    pack $activeVarsF.xF $activeVarsF.yF $activeVarsF.zF $activeVarsF.tF -side top -padx 10

##
    frame $middle.checksF 
    checkbutton $middle.checksF.sigma -text Sigma -variable sigmaV -command SetSigma

    frame $middle.checksF.repeatF
    checkbutton $middle.checksF.repeatF.repeatB -text Repeat -variable isRepeatV
    entry       $middle.checksF.repeatF.repeatE -width 3 -textvariable repeatIntervalV
    set repeatIntervalV 1
    pack $middle.checksF.repeatF.repeatB $middle.checksF.repeatF.repeatE -side left

    pack $middle.checksF.repeatF $middle.checksF.sigma -side bottom -anchor w

##
    label $middle.actionT -text "       Action"
    tk_optionMenu $middle.action actionV "Plot Image" "Fill Image" "Shade Image" "Show Data"

    pack $middle.listF $activeVarsF $middle.checksF -side left -anchor s
    pack $middle.actionT $middle.action -side left -anchor n

##############
    
    set bottom [frame $f.bottom ]
    button $bottom.ok -text "Go!"  -command Execute
    button $bottom.export -text "Export to Pic..." -command Export
    pack $bottom.ok $bottom.export -side right
    
##############
    pack $f.title $top $middle $bottom  -side top -fill x -pady 10

}

proc ActivateDatasetFile {listB} {
    global sent_string
    global ferret_output
    global typesList
    global title

    set nn [$listB curselection]
    if {$nn != ""} {
	set sent_string "cancel data/all\n"
	exp_send $sent_string
	ExpectProgramOutput 1

	set sent_string "use \"[$listB get $nn]\"\n"
	exp_send $sent_string
	ExpectProgramOutput 1

	set sent_string "show data\n"
	exp_send $sent_string
	ExpectProgramOutput 1

	#parse output with the help of awk
	set types [exec echo $ferret_output | awk { /^ [^ \t]/ { if (start==1) print $1; if ($1=="name") start=1}}]

	set types [split $types]

	$title config -text [$listB get $nn]
	$typesList delete 0 end
	foreach type $types {
	    if {$type != "..." && $type != "yes?" && $type != ""} {
		$typesList insert end $type
	    }
	}

    }
}

proc TypeChanged {} {
    global typesList
    global ferret_output
    global sigmaV
    global fromX toX fromY toY fromZ toZ fromT toT Xlimits Ylimits Zlimits Tlimits 

    set type [$typesList get [ $typesList curselection]]
    if {$sigmaV} {
	set type "$type\_on_z"
    }
    set sent_string "show grid $type\n"
    exp_send $sent_string
    ExpectProgramOutput 1

    set fromX 	""
    set toX 	""
    set fromY 	""
    set toY 	""
    set fromZ 	""
    set toZ 	""
    set fromT 	""
    set toT 	""

    #parse output with the help of awk
    set lower "??"
    set upper "??"
    set s [exec echo $ferret_output | awk { $1=="LON" {print $5}}]
    if {[scan $s "%f" x] == 1} {set lower $x}

    set s [exec echo $ferret_output | awk { $1=="LON" {print $6}}]
    if {[scan $s "%f" x] == 1} {set upper $x}
    
    set Xlimits [format "X %13s"  "$lower-$upper"]

    set lower "??"
    set upper "??"
    set s [exec echo $ferret_output | awk { $1=="LAT" {print $5}}]
    if {[scan $s "%f" x] == 1} {set lower $x}

    set s [exec echo $ferret_output | awk { $1=="LAT" {print $6}}]
    if {[scan $s "%f" x] == 1} {set upper $x}

    set Ylimits [format "Y %13s"  "$lower-$upper"]

    set lower "??"
    set upper "??"
    if { ! $sigmaV} {
	set s [exec echo $ferret_output | awk { $1=="SIGMA" {print $6}}]
	if {[scan $s "%f" x] == 1} {set lower $x}
	
	set s [exec echo $ferret_output | awk { $1=="SIGMA" {print $7}}]
	if {[scan $s "%f" x] == 1} {set upper $x}
    } else {
	set s [exec echo $ferret_output | awk { $1=="ZDEP" {print $6}}]
	if {[scan $s "%f" x] == 1} {set lower $x}
	
	set s [exec echo $ferret_output | awk { $1=="ZDEP" {print $7}}]
	if {[scan $s "%f" x] == 1} {set upper $x}
    }

    set Zlimits [format "Z %13s"  "$lower-$upper"]

    set lower "??"
    set upper "??"
    set s [exec echo $ferret_output | awk { $1=="TIME" {print $6}}]
    if {[scan $s "%f" x] == 1} {set lower $x}

    set s [exec echo $ferret_output | awk { $1=="TIME" {print $7}}]
    if {[scan $s "%f" x] == 1} {set upper $x}

    set Tlimits [format "T %13s"  "$lower-$upper"]
}    

proc Execute {} {
    global sent_string
    global activeWindow
    global zoomV
    global frameV prevFrameV
    global typesList
    global actionV
    global overlayV
    global sigmaV
    global isRepeatV repeatIntervalV
    global fromX toX fromY toY fromZ toZ fromT toT
    global errorMessage

    # to check if all the needed parameters are choosen

    if { $fromX == $toX } {
	set isDeltaX 0
    } else {
	set isDeltaX 1
    }

    if { $fromY == $toY } {
	set isDeltaY 0
    } else {
	set isDeltaY 1
    }
    if { $fromZ == $toZ } {
	set isDeltaZ 0
    } else {
	set isDeltaZ 1
    }
    if { $fromT == $toT } {
	set isDeltaT 0
    } else {
	set isDeltaT 1
    }
    
    set deltaNum [expr $isDeltaX + $isDeltaY + $isDeltaZ + $isDeltaT + $isRepeatV]
    
    switch $actionV {
	"Plot Image" {
	    if { $deltaNum == 1 } {
		set canPrint 1
	    } else {
		set canPrint 0
		set errorMessage "Can Plot only 1D fields"
	    }
	}
	"Shade Image" {
	    if { $deltaNum == 2 } {
		set canPrint 1
	    } else {
		set canPrint 0
		set errorMessage "Can Shade only 2D fields"
	    }
	}
	"Fill Image" {
	    if { $deltaNum == 2 } {
		set canPrint 1
	    } else {
		set canPrint 0
		set errorMessage "Can Fill only 2D fields"
	    }
	}
	"Show Data" {
	    set canPrint 1
	}
    }
    
    if { $canPrint } {
	# "Show data" does not need a window
	if {$actionV != "Show Data"} {
	    set sent_string "set w/size=$zoomV $activeWindow\n"
	    exp_send $sent_string
	    ExpectProgramOutput 1
	    
	    
	    if {$frameV != $prevFrameV && $frameV=="none"} {
		set sent_string "set v full\n"
		exp_send $sent_string
		ExpectProgramOutput 1
	    }
	    set prevFrameV $frameV
	    
	    if {$frameV != "none"} {
		set sent_string "set v $frameV\n"
		exp_send $sent_string
		ExpectProgramOutput 1
	    }
	}
	
	set nn [$typesList curselection]
	set type [$typesList get $nn]
	if {$sigmaV} {
	    set type "$type\_on_z"
	}
	
	
	##
	if {$fromX == $toX} {
	    set Xqualifier "x=$toX"
	} else {
	    set Xqualifier "x=$fromX:$toX"
	}
	
	if {$fromY == $toY} {
	    set Yqualifier "y=$toY"
	} else {
	    set Yqualifier "y=$fromY:$toY"
	}
	
	if {$fromZ == $toZ} {
	    set Zqualifier "z=$toZ"
	} else {
	    set Zqualifier "z=$fromZ:$toZ"
	}
	
	if {$fromT == $toT} {
	    set Tqualifier "t=$toT"
	} else {
	    set Tqualifier "t=$fromT:$toT"
	}
	
	if {$overlayV} {
	    set overlayQualifier "/overlay"
	} else {
	    set overlayQualifier ""
	}
	
	##
	if {! $isRepeatV} {
	    switch $actionV {
		"Plot Image" {
		    set sent_string "plot $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier/$Tqualifier $type\n"
		}
		"Fill Image"  {
		    set sent_string "fill/key/line $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier/$Tqualifier $type\n"
		}
		"Shade Image" {
		    set sent_string "shade $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier/$Tqualifier $type\n"
		}
		"Show Data" {
		    set sent_string "list /$Xqualifier/$Yqualifier/$Zqualifier/$Tqualifier $type\n"
		}
	    }
	} else {
	    switch $actionV {
		"Plot Image" {
		    set sent_string "repeat/$Tqualifier:$repeatIntervalV plot $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier $type\n"
		}
		"Fill Image"  {
		    set sent_string "repeat/$Tqualifier:$repeatIntervalV fill/key/line $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier $type\n"
		}
		"Shade Image" {
		    set sent_string "repeat/$Tqualifier:$repeatIntervalV shade $overlayQualifier/$Xqualifier/$Yqualifier/$Zqualifier $type\n"
		}
		"Show Data" {
		    set sent_string "repeat/$Tqualifier:$repeatIntervalV list /$Xqualifier/$Yqualifier/$Zqualifier $type\n"
		}
	}
	}
	
	exp_send $sent_string
	ExpectProgramOutput 1
    } else {
	tk_messageBox -title "ERROR" -message $errorMessage -type ok
	#ErrorMessage
	return
    }

}


proc Export {} {
    global sent_string
    global activeWindow
    global zoomV

    set sent_string "set w/size=$zoomV $activeWindow\n"
    exp_send $sent_string
    ExpectProgramOutput 1

    set types {
	{"Gif files"		{.gif}	}
	{"All files"		*}
    }
    set file [tk_getSaveFile -title "Choose Name" -filetypes $types -initialdir . -defaultextension ".gif"] 
 
    set sent_string "frame/file=\"$file\"\n"
    exp_send $sent_string
    ExpectProgramOutput 1
}

proc RunScript {} {
    global sent_string
    global activeWindow
    global zoomV

    set types {
	{"Script files"		{.jnl}	}
	{"All files"		*}
    }
    set file [tk_getOpenFile -title "Choose Script" -filetypes $types -initialdir /usr/local/ferret/contrib] 
 
    set sent_string "go \"$file\"\n"
    exp_send $sent_string
    ExpectProgramOutput 1
}

proc SetSigma {} {
    global sigmaV
    if {$sigmaV} {
	set sent_string "go \"/usr/local/ferret/contrib/sigma_coord_trans.jnl\"\n"
	exp_send $sent_string
	ExpectProgramOutput 1
    }
    TypeChanged
}

proc ErrorMessage {} {

    global tk_strictMotif
    global alreadyGotError
    global errorMessage

    option add *font 10x20
    set tk_strictMotif 0
    
    if { !$alreadyGotError } {
    toplevel .werr
    set alreadyGotError 1
    
    wm title .werr "Error"
    wm resizable .werr 0 0
    tk appname Xmsg
    
    puts $errorMessage
    
    frame .werr.ferr -bg green
    pack .werr.ferr -side top -fill both -expand true
    
    set lf [frame .werr.ferr.label_frame]
    label $lf.l_msg -text $errorMessage -bg green -relief ridge -bd 5
    pack $lf.l_msg -fill both -expand true
    pack $lf -side top 
    
    grab .werr
    
    set bf [frame .werr.ferr.buttom_frame]
    button $bf.b_OK -bg green -text "OK" -command {wm withdraw .werr; grab release .werr}
    pack $bf.b_OK -fill both
    pack $bf -side bottom 
    } else {
	wm deiconify .werr
	label $lf.l_msg -text $errorMessage -bg green -relief ridge -bd 5
	pack $lf.l_msg -fill both -expand true
	pack $lf -side top 
	button $bf.b_OK -bg green -text "OK" -command {wm withdraw .werr; grab release .werr}
	pack $bf.b_OK -fill both
	pack $bf -side bottom 
	
    }

}



Start $argc $argv




